@@ -58,8 +58,7 @@ def mk_element_from_spec(spec: FrontElementSpec):
         factory = _spec.pop(ELEMENT_KEY)
     except KeyError:
         raise RuntimeError(
-            f'Key "{ELEMENT_KEY}" is missing in the following element specification: \
-                {spec}'
+            f'Key "{ELEMENT_KEY}" is missing in the following element specification: {spec}'
         )
     try:
         return factory(**_spec)
@@ -150,7 +149,7 @@ class InputBase(FrontComponentBase):
         super().__post_init__()
         if not isinstance(self.value, BoundData):
             value = self.value
-            self.value = self._create_bound_data(self.value_key)
+            self.value = self._create_bound_data(self.input_key)
             if self.value.get() is ValueNotSet and value is not ValueNotSet:
                 self.value.set(value)
         dflt_value = self.obj.default
@@ -164,22 +163,10 @@ class InputBase(FrontComponentBase):
         if self.on_value_change and value is not ValueNotSet:
             call_forgivingly(self.on_value_change, self.value.get())
 
-    # def pre_render(self):
-    #     super().pre_render()
-    #     if self.value.id != self.input_key:
-    #         new_value = getattr(b, self.input_key).get()
-    #         if new_value is not ValueNotSet and new_value != self.value.get():
-    #             self.value.set(new_value)
-    #             self._call_on_value_change()
-
     def post_render(self, render_result):
         self.value.set(render_result)
         return render_result
 
-    @property
-    def value_key(self) -> str:
-        return self._build_key('value')
-
     @property
     def view_key(self) -> str:
         return self._build_key('view')
@@ -192,27 +179,35 @@ class InputBase(FrontComponentBase):
         return f'{self.input_key}_{suffix}'
 
     @property
-    def _dflt_view_value(self) -> Any:
+    def _type(self):
+        return str
+
+    @property
+    def _dflt_view_value(self):
         return None
 
-    def _get_view_value(self):
-        return self.value.get() or self._dflt_view_value
+    def _get_init_view_value(self):
+        value = self.value.get()
+        if value in [None, ValueNotSet]:
+            return self._dflt_view_value
+        return self._type(value)
 
     def _init_view_value(self):
-        self.view_value = self._create_bound_data(self.view_key)
-        if self.view_value.get() is ValueNotSet:
-            self.view_value.set(self._get_view_value())
+        view_value = self._create_bound_data(self.view_key).get()
+        self.view_value = (
+            view_value if view_value is not ValueNotSet else self._get_init_view_value()
+        )
 
     def _init_none_value(self):
-        self.none_value = self._create_bound_data(self.none_key)
-        if self.none_value.get() is ValueNotSet:
-            self.none_value.set(self.value.get() is None)
+        none_value = self._create_bound_data(self.none_key).get()
+        self.none_value = (
+            none_value if none_value is not ValueNotSet else self.value.get() is None
+        )
 
     def _create_bound_data(self, key):
         if self.bound_data_factory is None:
             raise ValueError(
-                f'No factory provided to build a BoundData instance with id \
-                    {key}'
+                f'No factory provided to build a BoundData instance with id "{key}"'
             )
         return self.bound_data_factory(key)
 
@@ -301,8 +296,12 @@ class MultiSourceInputBase(InputBase):
             dict(
                 obj=self.obj,
                 name=k,
+                input_key=self.input_key,
                 value=self.value,
                 on_value_change=self.on_value_change,
+                bound_data_factory=self.bound_data_factory,
+                is_noneable=self.is_noneable,
+                disabled=self.disabled,
                 **v,
             )
             for k, v in kwargs.items()
@@ -333,6 +332,10 @@ class TextInputBase(InputBase):
 
 @dataclass
 class BooleanInputBase(InputBase):
+    @property
+    def _type(self):
+        return bool
+
     @property
     def _dflt_view_value(self):
         return False
@@ -348,6 +351,10 @@ class IntInputBase(NumberInputBase):
     min_value: int = None
     max_value: int = None
 
+    @property
+    def _type(self):
+        return int
+
     @property
     def _dflt_view_value(self):
         return 0
@@ -359,6 +366,10 @@ class FloatInputBase(NumberInputBase):
     max_value: float = None
     step: float = None
 
+    @property
+    def _type(self):
+        return float
+
     @property
     def _dflt_view_value(self):
         return 0.0
@@ -367,6 +378,7 @@ class FloatInputBase(NumberInputBase):
 @dataclass
 class FileUploaderBase(InputBase):
     type: Optional[Union[str, List[str]]] = None
+    accept_multiple_files: bool = False
 
 
 SELECT_BOX_DFLT_INDEX = 0
@@ -374,14 +386,11 @@ SELECT_BOX_DFLT_INDEX = 0
 
 @dataclass
 class SelectBoxBase(InputBase):
-    options: Union[Iterable, BoundData] = None
+    options: Union[Iterable, Callable] = None
 
     def __post_init__(self):
         super().__post_init__()
         self.options = self.options or []
-
-    def pre_render(self):
-        super().pre_render()
         options = self._ensure_options()
         if not options:
             annot = self.obj.annotation
@@ -389,18 +398,17 @@ class SelectBoxBase(InputBase):
                 options = list(get_args(annot))
         self._options = list(options)
         if self._options:
-            view_value = self.view_value.get()
             self._preselected_index = (
-                self._options.index(view_value)
-                if view_value in self._options
+                self._options.index(self.view_value)
+                if self.view_value in self._options
                 else SELECT_BOX_DFLT_INDEX
             )
             selected_value = self._options[self._preselected_index]
-            if selected_value != view_value:
-                self.view_value.set(selected_value)
+            if selected_value != self.view_value:
+                self.view_value = selected_value
 
     @property
-    def _dflt_view_value(self) -> Any:
+    def _dflt_view_value(self):
         options = self._ensure_options()
         return options[SELECT_BOX_DFLT_INDEX] if options else None
 
@@ -411,19 +419,18 @@ class SelectBoxBase(InputBase):
 @dataclass
 class KwargsInputBase(InputBase):
     inputs: dict = None
-    func_sig: Sig = None
+    func_sig: Union[Sig, Callable] = None
 
-    def __post_init__(self):
-        super().__post_init__()
+    def pre_render(self):
+        super().pre_render()
+        func_sig = self.func_sig if isinstance(self.func_sig, Sig) else self.func_sig()
+        func_sig = func_sig or Sig()
 
-        @self.func_sig
+        @func_sig
         def get_kwargs(**kwargs):
             return kwargs
 
         self.get_kwargs = get_kwargs
 
-    def _get_kwargs(self, **kwargs):
-        return kwargs
-
     def _return_kwargs(self, output):
         self.value.set(output)
